<?php
namespace Service\Mongo\Types;


abstract class Type {

	const ID = 'id';
	const DATE = 'date';
	const DEFAULT_TYPE = 'DEFAULT';

	private static $typeObjects = array();
	private static $typesMap = array(
		self::ID => 'Service\Mongo\Types\IdType',
		self::DATE => 'Service\Mongo\Types\DateType',
		self::DEFAULT_TYPE => 'Service\Mongo\Types\DefaultType'
	);

	abstract public function convertToDatabaseValue($value);

	abstract public function convertToPHPValue($value);

	/**
	 * @param $type
	 *
	 * @return \Service\Mongo\Types\Type
	 */
	public static function getType($type) {
		if (!isset(self::$typesMap[$type])) {
			$type = self::DEFAULT_TYPE;
		}
		if (!isset(self::$typeObjects[$type])) {
			$className = self::$typesMap[$type];
			self::$typeObjects[$type] = new $className;
		}
		return self::$typeObjects[$type];
	}

	protected static function getTypeFromPHPVariable($variable) {
		if (is_object($variable)) {
			if ($variable instanceof \MongoDB\BSON\UTCDateTime) {
				return self::getType('date');
			} elseif ($variable instanceof \MongoDB\BSON\ObjectId) {
				return self::getType('id');
			}
		}
		return null;
	}

	protected static function getTypeFromDatabase($variable) {
		if (is_object($variable)) {
			if ($variable instanceof \MongoDB\BSON\UTCDateTime) {
				return self::getType('date');
			} elseif ($variable instanceof \MongoDB\BSON\ObjectId) {
				return self::getType('id');
			}
		}
		return null;
	}

	public static function convertDatabaseValueToPHP($value) {
        $type = self::getTypeFromDatabase($value);
        if ($type !== null) {
			/** @var Type $type */
			return $type->convertToPHPValue($value);
		}
		return $value;
	}

	public static function convertPHPToDatabaseValue($value) {
		$type = self::getTypeFromPHPVariable($value);
		if ($type !== null) {
			/** @var Type $type */
			return $type->convertToDatabaseValue($value);
		}
		return $value;
	}
}
