<?php


namespace Service;


use Doctrine\ORM\EntityManager;
use Exception;
use Service\Integration\IntegrationSM;
use Teknisa\Libs\Util\Masks\Picture;
use Teknisa\Model\Teknisa\Entities\CpCall;
use Teknisa\Model\Teknisa\Entities\Orcasolicita;
use Teknisa\Model\Teknisa\Entities\Parcsoliserv;
use Teknisa\Model\Teknisa\Entities\Tarefa;
use Util\Instance\Repository;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;

class Soliserv
{
    const ACTIVE = 'A';
    const IS_ISSUE = 'Y';
    const IS_TASK = 'Y';
    const FIRST_COMMENTS = 'Abertura de issue.';
    /**
     * @var EntityManager
     */
    public $entityManager;
    /**
     * @var Utilities
     */
    public $environment;

    protected $filial = '9999';

    protected $cdprocessoRastrea = array('0000010702','0000010703','0000010708','0000010709');
    /**
     * @var Teknisa\NewCode
     */
    private $newCodeService;
    /**
     * @var int|mixed
     */
    private $nrorg;

    public function __construct()
    {
        $this->entityManager  = InstanceProvider::getEntityManager();
        $this->environment    = InstanceProvider::getEnvironment();

        $this->nrorg = $this->environment->getCurrentOrganizationId();
    }

    public function findBudgetPlots($issueId,$namePlots)
    {
        return InstanceProvider::getRepository()->getBudget()
            ->findIssueBudgetPlots($issueId,$namePlots);
    }

    public function findBudget($SoliId,$nameProposal,$nameDiscount,$namePercent)
    {
        return InstanceProvider::getRepository()->getLocalSoliserv()
            ->findBudget($SoliId,$nameProposal,$nameDiscount,$namePercent);
    }

    public function findMaintenanceCost($nrsoliserv)
    {
        $budgetRepository = InstanceProvider::getRepository()->getLocalSoliserv();
        $organizationId = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $maintenance = $budgetRepository->MaintenanceCost($organizationId);
        $result = array();
        if($maintenance){
            $result['MAINTENANCE_PERCENTAGE'] = $maintenance;
            $Totalvalue = $budgetRepository->findManutSoliservBudget($nrsoliserv);
            $finalPrice = round($Totalvalue, 2);
            $maintenancePrice = round($finalPrice * $maintenance / 100, 2);
            $maintenancePrice = number_format($maintenancePrice, 2, '.', '');

            $maintenanceDiscounts = $budgetRepository->findManutSoliservDiscount($nrsoliserv);
            if($maintenanceDiscounts && $maintenanceDiscounts[0]){
                $result['VALUE_DISCOUNT'] = $maintenanceDiscounts[0]['MAINTENANCE_VALUE'];
                $result['PERCENTAGE_DISCOUNT'] =  $maintenanceDiscounts[0]['MAINTENANCE_PERCENT'];
                $result['TOTAL_VALUE'] = $maintenancePrice - $maintenanceDiscounts[0]['MAINTENANCE_VALUE'];
            }

            $result['MANUT_VALUE'] = $maintenancePrice;
        }else{
            $result['MANUT_VALUE'] = 0;
            $result['TOTAL_VALUE'] = 0;
            $result['PERCENTAGE_DISCOUNT'] = 0;
            $result['VALUE_DISCOUNT'] = 0;
            $result['MAINTENANCE_PERCENTAGE'] = 2;
        }
        $result['VALIDATE_EDIT']  = true;
        return $result;
    }

    public function findSoliBudgetPlots($issueId,$namePlots)
    {
        return InstanceProvider::getRepository()->getLocalSoliserv()
            ->findSoliBudgetPlots($issueId,$namePlots);
    }

    public function soliGenerateProposal($nrsoliserv)
    {
        $cdfaseOutProposal = IntegrationSM::integrationSMCreateProposalByNrsoliserv($nrsoliserv);
    }

    public function soliGenerateProposalValidate($nrsoliserv)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
        $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        $validate = 'OK';
        $type = 'OK';
        $cdfaseInProposal = Utilities::setSelectMultiInQuery(Utilities::getCdfaseTramInGenerateProposal(), false, "','");
        if (in_array($soliserv->getCdfasetram(), $cdfaseInProposal)) {
            $validate = 'FASE_ERROR';
            $type = 'ERROR';
        } else {
            $proposal = InstanceProvider::getRepository()->getLocalSoliserv()->findProposaByNrsoliserv($nrsoliserv);
            if (!empty($proposal)) {
                $validate = 'PRO_CRE_ALE';
                $type = 'ALERT';
            }
        }
        return ['validate' => $validate,'type'=>$type];
    }

    public function soliBudgetPlotsSave($rows,$nrsoliserv){

        $connection = InstanceProvider::getEntityManager()->getConnection();
        try {
            $connection->beginTransaction();

            $instanceRepository = InstanceProvider::getRepository();
            $entityManager = InstanceProvider::getEntityManager();
            $factoryBudgetPlots = InstanceProvider::getFactory()->getSoliserv();
            /** @var \Teknisa\Model\Teknisa\Repositories\Parcsoliserv $repositoryBudgetPlots */
            $repositoryBudgetPlots = $instanceRepository->getParcsoliserv();

            /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
            $soliserv = $instanceRepository->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));

            $lastParcel = 0;

            foreach ($rows as $row) {
                $orderPlots = intval($row['ORDER_PLOTS_NUM']);
                if($lastParcel < $orderPlots){
                    $lastParcel = $orderPlots;
                }
                $valuePlots = $row['VALUE_PLOTS'];
                /** @var Parcsoliserv $budgetPlots */
                $budgetPlots = $repositoryBudgetPlots->findOneBy(array('nrsoliserv' => $nrsoliserv,'nrparcsoli' => $orderPlots));
                if($budgetPlots){
                    $budgetPlots = $factoryBudgetPlots->updateParcsoliserv($budgetPlots,$orderPlots,$valuePlots);
                }else{
                    $budgetPlots = $factoryBudgetPlots->createParcsoliserv($soliserv,$orderPlots,$valuePlots);
                }
                $entityManager->persist($budgetPlots);
            }
            $entityManager->flush();
            if($lastParcel){
                $budgetPlotsIdToDelete = $instanceRepository->getLocalSoliserv()->findBudgetPlotsToDelete($nrsoliserv,$lastParcel);
                $budgetPlotsToDelete = $repositoryBudgetPlots->findBy(array('nrsoliserv' => $nrsoliserv,'nrparcsoli' => $budgetPlotsIdToDelete));
                foreach ($budgetPlotsToDelete as $parcelToDelete) {
                    $entityManager->remove($parcelToDelete);
                }
                if(!empty($budgetPlotsToDelete)){
                    $entityManager->flush();
                }
            }
            $connection->commit();
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }


    public function SoliBudgetSave($data){
        if(isset($data[0]['changed']) && $data[0]['changed']){
            $rows = [];
            foreach ($data as $datum) {
                if(!empty($datum['NRSOLISERV'])){
                    unset($datum['groupHeaders']);
                    $rows[] = $datum;
                }
            }
            $this->soliBudgetsaveItens($rows);
        }
    }

    public function soliBudgetRowSave($nrsoliserv, $budgetItemId, $proposalQuantity, $proposalValue)
    {

        $entityManager = InstanceProvider::getEntityManager();
        $instanceRepository = InstanceProvider::getRepository();
        $instanceFactory = InstanceProvider::getFactory();
        /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
        $soliserv = $instanceRepository->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
        if ($budgetItemId == '999999XXXX') {
            $proposalQuantity = abs($proposalQuantity);
            $proposalValue = abs($proposalValue);
            if ($proposalQuantity > 100) {
                $proposalQuantity = 100;
            }
            $soliserv->setVrpedesorcam($proposalQuantity);
            $soliserv->setVrdescorcam($proposalValue);
            $entityManager->persist($soliserv);
            $entityManager->flush();
        } else {
            $filterBudgetItem = array(
                'nrsoliserv' => $nrsoliserv,
                'cdfilial' => $soliserv->getCdfilial(),
                'nrorg' => $soliserv->getNrorg(),
                'cditorcaprod' => $budgetItemId,
            );
            /** @var Orcasolicita $orcasolicita  */
            $orcasolicita  = $instanceRepository->getOrcasolicita()->findOneBy($filterBudgetItem);
            $orcasolicita = $instanceFactory->getSoliserv()->saveOrcasolicita($orcasolicita,$soliserv);
            if ($proposalQuantity) {
                $orcasolicita ->setQtitorcsolic($proposalQuantity);
                $orcasolicita ->setVrorcadosoli($proposalValue);
                $orcasolicita = $instanceFactory->getSoliserv()->setOrcasolicitaEstimatValues($orcasolicita);
                $entityManager->persist($orcasolicita);
                $entityManager->flush();
            }
        }
    }

    public function SoliBudgetsaveItens($rows){

        $entityManager = InstanceProvider::getEntityManager();
        $instanceRepository = InstanceProvider::getRepository();
        $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $budgetId = null;
            $TotalProposalValue = 0;
            foreach ($rows as $row) {
                $nrsoliserv = $row['NRSOLISERV'];
                $budgetItemId = $row['BUDGET_ITEM_ID'];
                $proposalQuantity = $row['PROPOSAL_QUANTITY'];
                $proposalValue = $row['PROPOSAL_VALUE'];
                $TotalProposalValue += $proposalValue;
                $this->SoliBudgetRowSave($nrsoliserv, $budgetItemId,
                    $proposalQuantity, $proposalValue);
            }

            $repositoryBudgetPlots = $instanceRepository->getParcsoliserv();
            /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
            $soliserv = $instanceRepository->getSoliserv()->findOneBy(array('nrsoliserv' => $nrsoliserv));
            $budgetPlotsToDelete = $repositoryBudgetPlots->findBy(array('nrsoliserv' => $nrsoliserv));
            foreach ($budgetPlotsToDelete as $parcelToDelete) {
                $entityManager->remove($parcelToDelete);
            }
            if(!empty($budgetPlotsToDelete)){
                $entityManager->flush();
            }

            $budgetPlotInitial = InstanceProvider::getFactory()->getSoliserv()->createParcsoliserv($soliserv,1,$TotalProposalValue);
            $entityManager->persist($budgetPlotInitial);
            $entityManager->flush();
            $connection->commit();
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveMaintenanceCost($row)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($row['Soli_ID']) {
                /** @var \Teknisa\Model\Teknisa\Entities\Soliserv $soliserv */
                $soliserv = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $row['Soli_ID']));

                $valueDiscount = str_replace(",", ".", $row['VALUE_DISCOUNT']);
                $valueManut = str_replace(",", ".", $row['MANUT_VALUE']);
                $valueDiscount = floatval(preg_replace('/[^\d\.]+/', '', $valueDiscount));
                $valuePercent = str_replace(",", ".", $row['PERCENTAGE_DISCOUNT']);
                if($valueManut < $valueDiscount){
                    $valueDiscount = $valueManut;
                }
                if($valuePercent > 100){
                    $valuePercent = 100;
                }
                $soliserv->setVrpedesmanut($valuePercent);
                $soliserv->setVrdescmanut($valueDiscount);
                $entityManager->persist($soliserv);
                $entityManager->flush();
                $connection->commit();
            }
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    public function findTarefa($nrsoliserv){
        return InstanceProvider::getRepository()->getSoliservLocal()->findTarefa($nrsoliserv);
    }

    public function findItemTarefa($nrtarefa,$nrsoliserv){
        return InstanceProvider::getRepository()->getSoliservLocal()->findItemTarefa($nrtarefa,$nrsoliserv);
    }

    public function findAddItemTarefa(){
        return InstanceProvider::getRepository()->getSoliservLocal()->findAddItemTarefa();
    }

    public function findValidadeAtividateGerada($nrtarefas,$cdprocessos,$cditemtrabalhos){
        $cdfilial = $this->filial;
        return InstanceProvider::getRepository()->getSoliservLocal()->findValidadeAtividateGerada($cdfilial,$nrtarefas,$cdprocessos,$cditemtrabalhos);
    }

    public function findProcessTotalized($nrtarefa){
        $cdfilial = $this->filial;
        $return = InstanceProvider::getRepository()->getSoliservLocal()->findProcessTotalized($cdfilial,$nrtarefa);
        $picture = $this->getPicture();
        foreach ($return['data'] as &$item) {
            $item['ID_TREE'] = $picture->getCodigoComMascara($item['ID_TREE']);
        }
        return $return;
    }

    public function findPlanativtare($nrtarefa){
        $cdfilial = $this->filial;
        return InstanceProvider::getRepository()->getSoliservLocal()->findPlanativtare($cdfilial,$nrtarefa);
    }

    public function generateActivitiesSoliserv($nrtarefa, $cdarvprocess,$nrsoliserv, $processos, $itemtrabalhos){
        $connection           = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $this->newCodeService = InstanceProvider::getService()->getTeknisaNewCode();
            $soliservLocal = InstanceProvider::getRepository()->getSoliservLocal();
            $factoryPlativtare = InstanceProvider::getFactory()->getPlanativtare();
            $factoryDescplativ = InstanceProvider::getFactory()->getDescplativ();
            $factoryPlAtvRelItTr= InstanceProvider::getFactory()->getPlAtvRelItTr();
            $currentCdoperador = $this->environment->getCurrentCdoperador();
            $dtincativtar = new \DateTime();
            $cdoperincl = $currentCdoperador;
            $nrcplxitativ = 0;
            foreach ($processos as $process) {
                $cdprocesso = $process["CDPROCESSO"];
                list($nrplativtare, $cdfilial, $process, $cdoperresp, $activity) =
                    $this->insetBeforeForItem($process, $itemtrabalhos, $factoryPlativtare, $nrtarefa,
                        $cdprocesso, $dtincativtar, $cdoperincl, $nrcplxitativ);

                foreach ($itemtrabalhos as $itemtrabalho) {

                    $valorComplexidade = $this->getValorComplexidade($process, $itemtrabalho);

                    if ($process['IDGERATVCONS'] === 'S') {
                        $activity->setNrcplxitativ($activity->getNrcplxitativ() + $valorComplexidade);
                        $this->entityManager->persist($activity);
                        $this->entityManager->flush();
                    } else {
                        list($nrplativtare, $cdfilial, $process, $cdoperresp, $activity, $qttemplaitar)
                            = $this->insertPLanAtiv($itemtrabalho, $process,
                            $factoryPlativtare, $nrtarefa, $cdprocesso, $dtincativtar, $cdoperincl, $valorComplexidade);
                        $this->entityManager->persist($activity);
                        $this->entityManager->flush();

                        list($description, $itemtrabalho) = $this->getDescription($itemtrabalho, $soliservLocal,
                            $cdarvprocess, $nrsoliserv, $cdfilial, $nrtarefa);
                        if (!empty($description)) {
                            $descplativ = $factoryDescplativ->create($cdfilial, $nrtarefa, $nrplativtare, $description,$this->nrorg);
                            $this->entityManager->persist($descplativ);
                        }
                    }
                    $nrsequittare = $itemtrabalho['NRSEQUITTARE'];
                    $plAtvRelItTr = $factoryPlAtvRelItTr->create($cdfilial, $nrtarefa, $nrplativtare, $nrsequittare);
                    $this->entityManager->persist($plAtvRelItTr);
                    $this->entityManager->flush();
                }

                $idgeratvcons = $process['IDGERATVCONS'];
                if ($idgeratvcons === 'S') {
                    $this->entityManager->refresh($activity);
                    list($vrprodmedes, $pesoCargo, $vrpesoativgrc, $qttempminativ, $qttemplaitar) =
                        $this->setActivity($activity, $process, $soliservLocal, $cdoperresp);
                    $this->entityManager->persist($activity);
                }

                $this->setsRastreabilidade($cdprocesso, $soliservLocal, $cdfilial, $nrtarefa, $activity, $process,
                    $cdoperresp);
                $this->entityManager->flush();

            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            try{
                $connection->rollBack();
                throw $e;
            }catch (\Exception $e2) {}
            throw $e;
        }
        return[1];
    }

    public function findTarefaAllResponsavel($nrsoliserv){
        return InstanceProvider::getRepository()->getSoliservLocal()->findTarefaAllResponsavel($nrsoliserv);
    }

    public function findTarefaAssunto(){
        return InstanceProvider::getRepository()->getSoliservLocal()->findTarefaAssunto();
    }

    public function getParFilialGep($filial = null){
        if(empty($filial)){
            $filial = $this->filial;
        }
        return InstanceProvider::getRepository()->getSoliservLocal()->getParFilialGep($filial);
    }

    public function getPicture(){
        return new Picture($this->environment->getCurrentOrganizationId(),'CDPICTPROCES');
    }

    public function findProcessGeneration(){
        $parFilialGep = $this->getParFilialGep();
        $cdsetor = $parFilialGep['CDSETORPROC'];
        $cdnvprocesso = $parFilialGep['CDNVPESQPROC'];
        $return = InstanceProvider::getRepository()->getSoliservLocal()->findProcessGeneration($cdsetor, $cdnvprocesso);
        $picture = $this->getPicture();
        foreach ($return['data'] as &$item) {
            $item['ID_TREE'] = $picture->getCodigoComMascara($item['ID_TREE']);
        }
        return $return;
    }

    public function findActivitiesGeneration($cdarvprocess){
        $picture = $this->getPicture();
        $cdnvprocesso = $picture->getNumeroDeNiveis();
        if(empty($cdarvprocess)){
            $cdarvprocess = 9999999;
        }
        return InstanceProvider::getRepository()->getSoliservLocal()->findActivitiesGeneration($cdarvprocess, $cdnvprocesso);
    }

    public function findDefaultTaskSoli($nrsoliserv){
        $currentCdoperador = $this->environment->getCurrentCdoperador();
        return InstanceProvider::getRepository()->getSoliservLocal()->findDefaultTaskSoli($nrsoliserv,$currentCdoperador);
    }

    public function saveTarefaSoli($nrsoliserv, $cdassunto, $cdoperresp, $dsrefetarefa, $idorigtare, $nrpriotarefa){
        $currentCdoperador = $this->environment->getCurrentCdoperador();
        $tarefa = InstanceProvider::getFactory()->getTarefa()->createTarefa($nrsoliserv, $cdassunto, $cdoperresp, $dsrefetarefa, $idorigtare, $nrpriotarefa,$currentCdoperador);
        $this->entityManager->persist($tarefa);
        $this->entityManager->flush();
    }

    public function editTarefaSoli($nrtarefa, $cdoperresp, $dsrefetarefa, $nrpriotarefa){
        $tarefa = InstanceProvider::getRepository()->getTarefa()->findOneBy(array('nrtarefa' => $nrtarefa));
        $tarefa = InstanceProvider::getFactory()->getTarefa()->editTarefa($tarefa, $cdoperresp, $dsrefetarefa, $nrpriotarefa);
        $this->entityManager->persist($tarefa);
        $this->entityManager->flush();
    }

    public function saveItemTarefaSoli($cditemtrab, $nrtarefa, $nrprioittare, $nrcplximplem, $nrcplxitemtb){
        $currentCdoperador = $this->environment->getCurrentCdoperador();
        /** @var Tarefa $tarefa */
        $tarefa = InstanceProvider::getRepository()->getTarefa()->findOneBy(array('nrtarefa' => $nrtarefa));
        $cdfilial = $tarefa->getCdfilial();
        $dsrefeittare = $tarefa->getDsrefetarefa();
        $nrsequittare = InstanceProvider::getRepository()->getSoliservLocal()->getNrsequittare($nrtarefa,$cdfilial);
        $parFilialGep = $this->getParFilialGep($cdfilial);
        $vrpesoaltcpx = $parFilialGep['VRPESOALTCPX'];
        $itemtarefa = InstanceProvider::getFactory()->getItemTarefa()->createItemTarefa($nrtarefa, $cditemtrab,$cdfilial,
            $nrsequittare,$dsrefeittare,$nrprioittare, $nrcplximplem, $nrcplxitemtb,$currentCdoperador,$vrpesoaltcpx);
        $this->entityManager->persist($itemtarefa);
        $this->entityManager->flush();
    }

    public function editItemTarefaSoli($nrsequittare, $nrtarefa, $nrprioittare, $nrcplximplem, $nrcplxitemtb){
        $currentCdoperador = $this->environment->getCurrentCdoperador();
        /** @var Tarefa $tarefa */
        $tarefa = InstanceProvider::getRepository()->getTarefa()->findOneBy(array('nrtarefa' => $nrtarefa));
        /** @var Tarefa $tarefa */
        $itemTarefa = InstanceProvider::getRepository()->getItemTarefa()->findOneBy(array('nrtarefa' => $nrtarefa, 'nrsequittare' => $nrsequittare));
        $cdfilial = $tarefa->getCdfilial();
        $dsrefeittare = $tarefa->getDsrefetarefa();
        $parFilialGep = $this->getParFilialGep($cdfilial);
        $vrpesoaltcpx = $parFilialGep['VRPESOALTCPX'];
        $itemtarefa = InstanceProvider::getFactory()->getItemTarefa()->editItemTarefa($itemTarefa ,$cdfilial,
            $nrsequittare,$dsrefeittare,$nrprioittare, $nrcplximplem, $nrcplxitemtb,$currentCdoperador,$vrpesoaltcpx);
        $this->entityManager->persist($itemtarefa);
        $this->entityManager->flush();
    }

    public function deleteTarefaSoli($nrtarefa){
        $tarefa = InstanceProvider::getRepository()->getTarefa()->findOneBy(array('nrtarefa' => $nrtarefa));
        $this->entityManager->remove($tarefa);
        $this->entityManager->flush();
    }

    public function deleteItemTarefasSoli($nrtarefa, $nrsequittare){
        $itemTarefa = InstanceProvider::getRepository()->getItemTarefa()->findOneBy(array('nrtarefa' => $nrtarefa,'nrsequittare' => $nrsequittare));
        $this->entityManager->remove($itemTarefa);
        $this->entityManager->flush();
    }

    public function getPageRequisite($nrsoliserv)
    {
        InstanceProvider::getService()->getAttachment()->getPageRequisite($nrsoliserv);
    }

    public function insertFileSpecificationTaskS($nrsoliserv, $document)
    {
        InstanceProvider::getService()->getAttachment()->insertFileSpecificationTaskS($nrsoliserv, $document);
    }

    public function editGridActivityS($cdfilial, $nrtarefa, $nrplativtare, $nrprioativtr,
                                      $cdoperresp,$qthrinfativtar)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\Planativtare $planativtare */
        $planativtare = InstanceProvider::getRepository()->getPlanativtare()->findOneBy(array('nrtarefa' => $nrtarefa,'cdfilial' => $cdfilial,'nrplativtare' => $nrplativtare));
        $planativtare->setNrprioativtr($nrprioativtr);
        $planativtare->setcdoperresp($cdoperresp);
        $planativtare->setQthrinfativtar($qthrinfativtar);
        $this->entityManager->persist($planativtare);
        $this->entityManager->flush();
    }

    public function deleteGridActivityS($nrtarefa, $cdfilial, $nrplativtare){
        /** @var \Teknisa\Model\Teknisa\Entities\Descplativ[] $descplativ */
        $descplativs = InstanceProvider::getRepository()->getDescplativ()->findBy(array('nrtarefa' => $nrtarefa,'cdfilial' => $cdfilial,'nrplativtare' => $nrplativtare));
        foreach ($descplativs as $descplativ) {
            $this->entityManager->remove($descplativ);
        }
        /** @var \Teknisa\Model\Teknisa\Entities\Platvrelittr[] $planativtare */
        $plAtvRelItTrs = InstanceProvider::getRepository()->getPlAtvRelItTr()->findBy(array('nrtarefa' => $nrtarefa,'cdfilial' => $cdfilial,'nrplativtare' => $nrplativtare));
        foreach ($plAtvRelItTrs as $plAtvRelItTr) {
            $this->entityManager->remove($plAtvRelItTr);
        }
        /** @var \Teknisa\Model\Teknisa\Entities\Planativtare $planativtare */
        $planativtare = InstanceProvider::getRepository()->getPlanativtare()->findOneBy(array('nrtarefa' => $nrtarefa,'cdfilial' => $cdfilial,'nrplativtare' => $nrplativtare));
        $this->entityManager->remove($planativtare);
        $this->entityManager->flush();
    }

    /**
     * @param $process
     * @param $itemtrabalho
     * @return float|mixed
     */
    public function getValorComplexidade($process, $itemtrabalho)
    {
        if ($process['IDREFECPLX'] === 'A') {
            $valorComplexidade = $itemtrabalho['NRCPLXIMPLEM'] + round($process, 2);
        } else {
            $valorComplexidade = $itemtrabalho['NRCPLXITTARE'] + round($process, 2);
        }
        return $valorComplexidade;
    }

    /**
     * @param $nrtarefa
     * @return String
     */
    public function getPrxPlanAtivTare($nrtarefa): string
    {
        $nrOrg = $this->environment->getCurrentOrganizationId();
        $cdfilial = $this->filial;
        return $this->newCodeService->getCodeByTableName('PLANATIVTARE'.$cdfilial.$nrtarefa, $nrOrg,1,5);
    }

    /**
     * @param $process
     * @param $itemtrabalhos
     * @param \Factory\Planativtare $factoryPlativtare
     * @param $nrtarefa
     * @param $cdprocesso
     * @param \DateTime $dtincativtar
     * @param $cdoperincl
     * @param int $nrcplxitativ
     * @return array
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function insetBeforeForItem($process,  $itemtrabalhos, \Factory\Planativtare $factoryPlativtare,
                                       $nrtarefa, $cdprocesso, \DateTime $dtincativtar, $cdoperincl, int $nrcplxitativ): array
    {
        $cdoperresp = null;
        $activity = null;
        $nrplativtare = null;
        $cdfilial = $itemtrabalhos[0]["CDFILIAL"];
        if ($process['IDGERATVCONS'] === 'S') {
            $nrplativtare = $this->getPrxPlanAtivTare($nrtarefa);

            $cdoperresp = empty($process["CDOPERADOR"]) ? $itemtrabalhos[0]["CDOPERADOR"] : $process["CDOPERADOR"];

            /** @var \Teknisa\Model\Teknisa\Entities\Planativtare $activity */
            $activity = $factoryPlativtare->create($nrplativtare, $cdfilial, $nrtarefa, $cdprocesso, $cdoperresp,
                $dtincativtar, $cdoperincl, $nrcplxitativ);
            $this->entityManager->persist($activity);
            $this->entityManager->flush();
            $this->entityManager->refresh($activity);
        }
        return array($nrplativtare, $cdfilial, $process, $cdoperresp, $activity);
    }

    /**
     * @param $itemtrabalho
     * @param $process
     * @param \Factory\Planativtare $factoryPlativtare
     * @param $nrtarefa
     * @param $cdprocesso
     * @param \DateTime $dtincativtar
     * @param $cdoperincl
     * @param float $valorComplexidade
     * @return array
     */
    public function insertPLanAtiv( $itemtrabalho, $process, \Factory\Planativtare $factoryPlativtare, $nrtarefa, $cdprocesso, \DateTime $dtincativtar, $cdoperincl, float $valorComplexidade): array
    {
        $nrplativtare = $this->getPrxPlanAtivTare($nrtarefa);
        $cdfilial = $itemtrabalho["CDFILIAL"];
        $cdoperresp = empty($process["CDOPERADOR"]) ? $itemtrabalho["CDOPERADOR"] : $process["CDOPERADOR"];
        $activity = $factoryPlativtare->create($nrplativtare, $cdfilial, $nrtarefa, $cdprocesso, $cdoperresp,
            $dtincativtar, $cdoperincl, $valorComplexidade);
        $activity->setCditemtrab($itemtrabalho['CDITEMTRAB']);
        $activity->setNrcplxplanat($valorComplexidade);
        $qttemplaitar = (($valorComplexidade / $process['VRPRODMEDES']) + $process['QTTEMPMINATIV']);
        $activity->setQttemplaitar($qttemplaitar);
        if ($qttemplaitar > 0) {
            $activity->setNrcplxitativ(round(($qttemplaitar * $process['VRPRODMEDES']), 2));
            $activity->setNrcplxplanat($activity->getNrcplxitativ());
        }
        return array($nrplativtare, $cdfilial, $process, $cdoperresp, $activity, $qttemplaitar);
    }

    /**
     * @param $itemtrabalho
     * @param \Repository\Soliserv $soliservLocal
     * @param $cdarvprocess
     * @param $nrsoliserv
     * @param $cdfilial
     * @param $nrtarefa
     * @return array
     */
    public function getDescription($itemtrabalho, \Repository\Soliserv $soliservLocal, $cdarvprocess, $nrsoliserv, $cdfilial, $nrtarefa): array
    {
        $description = '';
        if (!empty($itemtrabalho['TXPLANITTARE'])) {
            $description = $itemtrabalho['TXPLANITTARE'];
        } else {
            $qryVerifCor = $soliservLocal->findQryVerifCor($cdarvprocess);
            $qryVerifVerif = $soliservLocal->findQryVerifVerif($nrsoliserv, $this->filial);
            if (!empty($qryVerifCor) && !empty($qryVerifVerif)) {
                $description = $qryVerifVerif[0]['TXREATIVTARE'];
            } else {
                $descTarefa = $soliservLocal->findqryDescTarefa($cdfilial, $nrtarefa);
                if (!empty($descTarefa)) {
                    $description = $descTarefa[0]['TXTAREFA'];
                }
            }
        }
        return array($description, $itemtrabalho);
    }

    /**
     * @param $activity
     * @param $process
     * @param \Repository\Soliserv $soliservLocal
     * @param $cdoperresp
     * @return array
     */
    public function setActivity($activity, $process, \Repository\Soliserv $soliservLocal, $cdoperresp): array
    {
        $activity->setNrcplxplanat($activity->getNrcplxitativ());
        $vrprodmedes = $process['VRPRODMEDES'];
        if ($vrprodmedes > 0) {
            $pesoCargo = $soliservLocal->findqryPesoCargo($cdoperresp);
            $vrpesoativgrc = $pesoCargo[0]['VRPESOATIVGRC'];
            $qttempminativ = $process['QTTEMPMINATIV'];
            $qttemplaitar = (($activity->getNrcplxplanat() / $vrprodmedes) * $vrpesoativgrc) + $qttempminativ;
            $activity->setQttemplaitar($qttemplaitar);
            if ($qttemplaitar > 0) {
                $activity->setNrcplxitativ(round(($activity->getQttemplaitar() * $vrprodmedes), 2));
                $activity->setNrcplxplanat(round(($activity->getQttemplaitar() * $vrprodmedes), 2));
            }
        } else {
            $activity->setQttemplaitar(0);
        }
        return array($vrprodmedes, $pesoCargo, $vrpesoativgrc, $qttempminativ, $qttemplaitar);
    }

    /**
     * @param $cdprocesso
     * @param \Repository\Soliserv $soliservLocal
     * @param $cdfilial
     * @param $nrtarefa
     * @param $activity
     * @param $process
     * @param $cdoperresp
     * @throws \Doctrine\ORM\ORMException
     */
    public function setsRastreabilidade($cdprocesso, \Repository\Soliserv $soliservLocal, $cdfilial, $nrtarefa, $activity, $process, $cdoperresp): void
    {
        if (in_array($cdprocesso, $this->cdprocessoRastrea)) {
            $qryRastreabilidade = $soliservLocal->findqryRastreabilidade($cdfilial, $nrtarefa);
            if (!empty($qryRastreabilidade)) {
                foreach ($qryRastreabilidade as $rastre) {
                    $activity->setNrcplxitativ($activity->getNrcplxitativ() + ($rastre['NRCPLXITTRAB'] * $process['VRFATOCPLX']));
                    $activity->setNrcplxplanat($activity->getNrcplxitativ());
                    $this->entityManager->persist($activity);
                }
                $vrprodmedes = $process['VRPRODMEDES'];
                if ($vrprodmedes > 0) {
                    $pesoCargo = $soliservLocal->findqryPesoCargo($cdoperresp);
                    $vrpesoativgrc = $pesoCargo[0]['VRPESOATIVGRC'];
                    $qttempminativ = $process;
                    $qttemplaitar = (($activity->getNrcplxplanat() / $vrprodmedes) * $vrpesoativgrc) + $qttempminativ;
                    $activity->setQttemplaitar($qttemplaitar);
                } else {
                    $activity->setQttemplaitar(0);
                }
            }
        }
    }


}
